@@ -8,7 +8,7 @@ NULL
 setOldClass(Classes = 'package_version')
 # TODO: error with setOldClass, check what is going on
 setClass("keras_training_history") 
-setOldClass(Classes = "keras.engine.sequential.Sequential")
+setClass("keras.engine.sequential.Sequential")
 
 setClassUnion(name = "MatrixOrNULL", members = c("matrix", "NULL"))
 setClassUnion(name = "ListOrNULL", members = c("list", "NULL"))
@@ -93,29 +93,28 @@ setClassUnion(
 #' The PropCellTypes Class
 #'
 #' The \code{\linkS4class{PropCellTypes}} class is a data storage class which
-#' contains information related to the cell type composition matrices used to
+#' contains information related to cell type composition matrices used to
 #' simulate mixed transcriptional profiles. This matrix is stored in the
-#' \code{prob.matrix} slot while the rest contains additional information
-#' generated during the process and required in subsequent steps.
+#' \code{prob.matrix} slot while the other slots contain additional information
+#' generated during the process and required for subsequent steps.
 #'
 #' See \code{?\link{genMixedCellProp}} function for information about how cell
-#' type composition matrices are generated. Plots about the cell type proportion
+#' type composition matrices are generated. Plots of cell type proportion
 #' distributions can be accessed using the \code{\link{showProbPlot}} function
 #' (see \code{?\link{showProbPlot}} for more details).
 #'
 #' @slot prob.matrix Matrix of cell type proportions to simulate mixed
 #'   transcriptional profiles.
-#' @slot cell.names Matrix containing the name of cells to be used to generate
-#'   the simulated mixed transcriptional profiles.
-#' @slot set.list List of cells ordered according to the cell type they belong
-#'   to.
+#' @slot cell.names Matrix containing cells used to generate the simulated 
+#'   mixed transcriptional profiles.
+#' @slot set.list List of cells sorted by cell type.
 #' @slot set Vector containing cell names present in the object.
-#' @slot method Vector containing the method by which cell type proportions were
+#' @slot method Vector indicating the method by which cell type proportions were
 #'   generated.
-#' @slot plots Plots showing the distribution of the cell type proportions. See
+#' @slot plots Plots showing cell type proportion distributions. See
 #'   \code{?\link{showProbPlot}} for more details.
-#' @slot type.data Character indicating the type of data contained: train or
-#'   test.
+#' @slot type.data Character indicating the type of data contained: 
+#'   \code{'train'} or \code{'test'}.
 #'
 #' @export PropCellTypes
 #'   
@@ -201,8 +200,8 @@ setMethod(
 #' \pkg{SpatialDDLS} provides a \code{saveRDS} generic function that transforms
 #' the R6 object containing the trained model into a native valid R object.
 #' Specifically, the model is converted into a list with the architecture of the
-#' network and the weights learned during training. That is the minimum
-#' information needed to use the model as predictor. If you want to keep the
+#' network and the weights learned during training, which is the minimum
+#' information needed to use the model as a predictor. If you want to keep the
 #' optimizer state, see \code{?\link{saveTrainedModelAsH5}}. If you want to
 #' store either the \code{\linkS4class{DeconvDLModel}} or the
 #' \code{\linkS4class{SpatialDDLS}} objects on disk as RDA files, see
@@ -222,7 +221,15 @@ setMethod(
 #' @slot test.deconv.metrics Performance of the model on test data by cell type.
 #'   This slot is generated by the \code{\link{calculateEvalMetrics}} function
 #'   (see \code{?\link{calculateEvalMetrics}} for more details).
-#'
+#' @slot interpret.gradients Gradients for interpretation. \pkg{SpatialDDLS} 
+#'   provides some functions to better understand prediction made by the model 
+#'   (see \code{?\link{interGradientsDL}} for more details). This slot is a list 
+#'   of either one or two elements: gradients of either the loss function or the 
+#'   predicted class with respect to the input variables using pure (only one 
+#'   cell type) mixed transcriptional profiles. These gradients can be 
+#'   interpreted as to what extent the model is using these variables to predict 
+#'   each cell type proportions.
+#'    
 #' @export DeconvDLModel
 #'   
 DeconvDLModel <- setClass(
@@ -234,7 +241,8 @@ DeconvDLModel <- setClass(
     test.pred = "MatrixOrNULL",
     cell.types = "character",
     features = "character",
-    test.deconv.metrics = "ListOrNULL"
+    test.deconv.metrics = "ListOrNULL",
+    interpret.gradients = "ListOrNULL"
   )
 )
 
@@ -249,7 +257,8 @@ setMethod(
     test.pred = NULL,
     cell.types = "-",
     features = "-",
-    test.deconv.metrics = NULL
+    test.deconv.metrics = NULL,
+    interpret.gradients = NULL
   ) {
     .Object@model <- model
     .Object@training.history <- training.history
@@ -258,6 +267,7 @@ setMethod(
     .Object@cell.types <- cell.types
     .Object@features <- features
     .Object@test.deconv.metrics <- test.deconv.metrics
+    .Object@interpret.gradients <- interpret.gradients
     return(.Object)
   }
 )
@@ -284,7 +294,7 @@ setMethod(
                  collapse = "\n"))
       if (!is.null(test.deconv.metrics(object)) ||
           length(test.deconv.metrics(object)) > 0) {
-        cat("\n  Performance evaluation over each sample: ")
+        cat("\n  Performance evaluation of each sample: ")
         cat(paste(names(test.deconv.metrics(object)[[2]]), collapse = " "))
       }
     }
@@ -302,7 +312,7 @@ setClassUnion("DeconvDLModelOrNULL", c("DeconvDLModel", "NULL"))
 
 #' The SpatialDDLS Class
 #'
-#' The \code{\linkS4class{SpatialDDLS}}object is the core of the
+#' The \code{\linkS4class{SpatialDDLS}} object is the core of the
 #' \pkg{SpatialDDLS} package. This object stores different intermediate data
 #' needed for the construction of new deconvolution models, the spatial
 #' transcriptomics profiles to be deconvoluted, and the predicted cell type
@@ -319,14 +329,14 @@ setClassUnion("DeconvDLModelOrNULL", c("DeconvDLModel", "NULL"))
 #' for the simulation of new single-cell profiles. \item
 #' \code{\linkS4class{SummarizedExperiment}} class for simulated mixed
 #' transcriptional profiles storage. \item \code{\linkS4class{PropCellTypes}}
-#' class for compositional cell type matrices. See
+#' class for composition cell type matrices. See
 #' \code{?\linkS4class{PropCellTypes}} for details. \item
-#' \code{\linkS4class{DeconvDLModel}} class to store the information related to
+#' \code{\linkS4class{DeconvDLModel}} class to store information related to
 #' deep neural network models. See \code{?\linkS4class{DeconvDLModel}} for
 #' details. }
 #'
-#' In order to provide a way to work with large amounts of data on
-#' RAM-constrained machines, we provide the possibility to use HDF5 files as
+#' In order to provide a way to work with large amounts of data in
+#' RAM-constrained machines, we provide the possibility of using HDF5 files as
 #' back-end to store count matrices of both real and simulated single-cell
 #' profiles by using the \pkg{HDF5Array} and \pkg{DelayedArray} classes from the
 #' homonymous packages.
@@ -348,7 +358,7 @@ setClassUnion("DeconvDLModelOrNULL", c("DeconvDLModel", "NULL"))
 #'   Count matrices can be stored as \code{HDF5Array} objects using HDF5 files
 #'   as back-end in case of RAM limitations.
 #' @slot trained.model \code{\linkS4class{DeconvDLModel}} object with
-#'   information related to the trained model. See
+#'   information related to the deconvolution model. See
 #'   \code{?\linkS4class{DeconvDLModel}} for more details.
 #' @slot deconv.spots Deconvolution results. It consists of a list where each
 #'   element corresponds to the results for each
@@ -419,8 +429,12 @@ setMethod(
   )
 }
 
-.sceShow <- function(sce) {
-  cat(" ", dim(sce)[1], "features and", dim(sce)[2], "cells\n")
+.sceShow <- function(sce, type) {
+  if (type == "spatial") {
+    cat(" ", dim(sce)[1], "features and", dim(sce)[2], "spots\n")
+  } else {
+    cat(" ", dim(sce)[1], "features and", dim(sce)[2], "cells\n")
+  }
   if (is.null(rownames(sce))) rownames.sce <- "---"
   else rownames.sce <- .selectSome(vec = rownames(sce), num = 6)
   if (identical(colnames(sce), character(0))) colnames.sce <- "---"
@@ -430,7 +444,7 @@ setMethod(
 }
 
 .bulkShow <- function(se) {
-  cat("   ", dim(se)[1], "features and", dim(se)[2], "samples\n")
+  cat("   ", dim(se)[1], "features and", dim(se)[2], "spots\n")
   if (dim(rowData(se))[2] == 0) rownames.se <- "---" 
   else rownames.se <- .selectSome(vec = rownames(rowData(se)), num = 6)
   if (identical(colnames(se), character(0))) colnames.se <- "---"
@@ -494,26 +508,26 @@ setMethod(
     }
     if (!is.null(object@single.cell.real)) {
       cat("Real single-cell profiles:\n")
-      .sceShow(object@single.cell.real)
+      .sceShow(object@single.cell.real, type = "single-cell")
     } else {
       cat("Real single-cell profiles:\n")
-      .sceShow(S4Vectors::DataFrame())
+      .sceShow(S4Vectors::DataFrame(), type = "single-cell")
     }
     if (!is.null(object@spatial.experiments)) {
       cat("Spatial experiments:\n")
       cat(" ", length(object@spatial.experiments), "experiments\n")
       if (length(object@spatial.experiments) < 3)
-        sapply(object@spatial.experiments, .sceShow)
+        sapply(object@spatial.experiments, .sceShow, type = "spatial")
     } else {
       cat("Spatial experiments:\n")
-      .sceShow(S4Vectors::DataFrame())
+      .sceShow(S4Vectors::DataFrame(), type = "spatial")
     }
     if (!is.null(object@zinb.params)) {
       .zinbModelShow(object@zinb.params@zinbwave.model)
     }
     if (!is.null(object@single.cell.simul)) {
       cat("Simulated single-cell profiles:\n")
-      .sceShow(object@single.cell.simul)
+      .sceShow(object@single.cell.simul, type = "single-cell")
     }
     if (!is.null(object@prob.cell.types)) {
       cat("Cell type composition matrices:\n")
@@ -554,7 +568,7 @@ setMethod(
 .onLoad <- function(libname, pkgname) {
   if (.isConda()) {
     tryCatch(
-      expr = reticulate::use_condaenv("spatialddls-env", required = TRUE),
+      expr = reticulate::use_condaenv("SpatialDDLS-env", required = TRUE),
       error = function(e) NULL
     )
   }
